"""
This module provides a wrapper for interacting with Pastebin.

The module contains the `PastebinAPI` class which offers convenient methods to access various
endpoints of the API using HTTP requests. It supports custom headers and proxies for
additional flexibility.
"""
import logging
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
from bs4 import BeautifulSoup
from .paste import Paste


logger = logging.getLogger()


class ApiRequestException(Exception):
    pass


class PastebinAPI:
    """
    Wrapper class for interacting with Pastebin.

    This class provides convenient methods to access various endpoints of the
    API using HTTP requests. It supports custom headers and proxies for
    additional flexibility.

    Args:
        headers (dict, optional): Dictionary containing customer headers to be sent with each request.
        proxies (dict, optional): Dictionary containing proxy settings to be used for requests.
        debug (bool, optional): Setting this to true will print `urllib3` debug information.
    """
    def __init__(self, headers=None, proxies=None, debug=False):
        self.base_url = 'https://pastebin.com'
        self.session = requests.Session()

        retries = Retry(total=3, backoff_factor=1, status_forcelist=[429, 500, 502, 503, 504])
        self.session.mount('http://', HTTPAdapter(max_retries=retries))
        self.session.mount('https://', HTTPAdapter(max_retries=retries))

        if headers:
            self.session.headers.update(headers)
        if proxies:
            self.session.proxies.update(proxies)

        if debug:
            if not logger.handlers:
                logging.basicConfig()

            logging.getLogger("urllib3").setLevel(logging.DEBUG)

    def __get(self, endpoint, params=None):
        url = self.base_url + endpoint

        try:
            response = self.session.get(url, params=params)
            response.raise_for_status()
        except requests.RequestException as e:
            raise ApiRequestException(f'Failed to execute GET request: {str(e)}')

        return response

    def get_public_paste_list(self):
        endpoint = '/archive'

        try:
            response = self.__get(endpoint)
        except ApiRequestException as e:
            logger.warning(e)
            return None

        soup = BeautifulSoup(response.text, 'html.parser')
        pastes_table = soup.select('div[data-key]')

        pastes = [self._parse_paste(paste_html) for paste_html in pastes_table]

        return pastes

    def get_paste(self, paste):
        endpoint = f'/{paste.href}'

        try:
            response = self.__get(endpoint)
        except ApiRequestException as e:
            logger.warning(e)
            return None

        # Get paste metadata
        soup = BeautifulSoup(response.text, 'html.parser')
        paste = self._parse_paste_metadata(soup, paste)

        # Get paste text
        paste = self._fetch_paste_text(paste)

        return paste

    def _parse_paste(self, paste_html):
        paste_title = paste_html.find('a').text
        paste_link = paste_html.find('a')['href'].lstrip('/')
        paste_lang = paste_html.find_all('a')[-1].text

        return Paste(href=paste_link, lang=paste_lang, title=paste_title)

    def _parse_paste_metadata(self, soup, paste):
        paste.author = soup.find('div', class_='username').a.text
        paste.pub_date = soup.find('div', class_='date').span.text
        paste.category = soup.find('span', title='Category').text.split('|')[1].strip()

        return paste

    def _fetch_paste_text(self, paste):
        endpoint = f'/raw/{paste.href}'
        try:
            response = self.__get(endpoint)
        except ApiRequestException as e:
            logger.warning(f'Could not fetch paste text: {str(e)}')
        else:
            paste.text = response.text
            paste.fetched = True

        return paste


class PastebinClient(PastebinAPI):
    def __init__(self, headers=None, proxies=None, debug=False):
        super().__init__(headers, proxies, debug)
        self.pastes = []

    def update_paste_list(self):
        fetched_pastes = set(self.get_public_paste_list())
        existing_pastes = set(self.pastes)

        new_pastes = fetched_pastes - existing_pastes
        self.pastes = list(existing_pastes.union(fetched_pastes))

        return list(new_pastes)

    def fetch_paste(self, paste_index):
        if paste_index >= len(self.pastes):
            return None
        return self.get_paste(self.pastes[paste_index])

    def get_paste_list(self):
        return self.pastes

    def total_pastes(self):
        return len(self.pastes)

    def get_pastes_by_language(self, lang):
        return [
            self.pastes[i] for i, paste in enumerate(self.pastes)
            if paste.lang.lower() == lang.lower()
        ]
